#
# inetc.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later..
#

require 'net/session'
require 'net/smtp'
require 'net/pop'

require 'amstd/extmod'
require 'amstd/must'
require 'amstd/bug'
require 'amstd/dispatch'


module EST


class INETclient

  def initialize( host, port )
    host.must String
    port.must Integer

    @session = session_type.new( host, port )
  end

  abstract :session_type


  def logged_in?
    @session.active?
  end

  def address
    @session.address
  end
  alias host address

  def port
    @session.port
  end

  def socket
    @session.socket
  end


  private


  def do_exec( *args )
    return if logged_in?
    @session.start( INETsocket, *args )
    if_loggingin
    do_login
    if_loggedin
    do_logout
    @session.finish
  end

  abstract :do_login
  abstract :do_logout

end



class INETsocket < Net::ProtocolSocket

  include Dispatchable

  def initialize( addr, port, pipe = nil )
    addr.must String
    port.must Integer
    super( addr, port, pipe )

    @pipe << "TCP connection to #{addr} opened" if @pipe
  end

  property :socket, TCPsocket, true, true

  event_handler :on_writing
  event_handler :on_write
  event_handler :on_writeall

  event_handler :on_reading
  event_handler :on_read
  event_handler :on_readall


  def writeline( str )
    str.must String
    super
  end


  private

  def do_write_beg
    super
    dispatch :on_writing
  end

  def do_write_do( arg )
    ret = super
    dispatch :on_write, ret
    return ret
  end

  def do_write_fin
    ret = super
    dispatch :on_writeall, ret
    return ret
  end


  public

  def read( len, &blk )
    do_read(blk){ super len }
  end

  def readuntil( targ, &blk )
    do_read(blk){ super targ }
  end
      
  def read_pendstr( ret = '', &blk )
    do_read(blk){ super ret }
  end

  def read_pendlist
    do_read(nil){ super }
  end


  private

  def do_read( block )
    @rlen = 0
    push_event( :on_read, block ) if block
    dispatch :on_reading
    ret = yield
    dispatch :on_readall
    pop_event( :on_read ) if block

    return ret
  end

  def fill_rbuf
    tmp = @socket.sysread( READ_BLOCK )
    @buffer << tmp
                                  #$stderr.puts ">>>sysread size=#{tmp.size}"
    dispatch :on_read, tmp.size
  end

end



class CSMTPclient < INETclient

  def initialize( rc )
    addr = rc['host']
    port = (rc['port'] || 25).to_i
    @hello_domain = rc['hello-domain'] || ENV['HOSTNAME']
    @mvto         = rc['move-to']

    super addr, port
  end

  def session_type
    Net::SMTPSession
  end


  def exec
    if CORE.mailset.draft.size == 0 then
      if_nosendmail
      return
    end

    do_exec
  end


  private


  def if_nosendmail
  end

  def if_loggedin
    CORE.mailset.draft.each_with_index do |port,idx|
      mail = Mail.new( port )
      fr = fromaddr( mail )
      to = toaddrs( mail )
      str = CORE.composer.mailtext( mail, client )
      @session.sendmail( str, fr, to )
      if_sendmail( mail, idx )
    end
  end

  def if_loggedout
    return unless @mvto

    (0 .. CORE.mailset.draft.size - 1).each do |i|
      CORE.mailset.mvmail CORE.mailset.draft, @mvto, i
    end
  end

  def if_sendmail( mail, idx )
  end


  ###


  def fromaddr( mail )
    faddr = mail.from
    unless faddr then
      raise ArgumentError, "no 'from' address designation in mail"
    end

    faddr
  end

  def toaddrs( mail )
    toaddr = []
    mail.each_dest do |addr|
      toaddr.push addr.addr
    end
    if toaddr.size == 0 then
      raise ArgumentError,
        "no 'to' address designation in mail"
    end

    toaddr
  end

  def mailtext( mail, client )
    mail.body = NKF.nkf( '-j', mail.body )
    mail.to_s( "\r\n" )
  end

end


class SMTPclient < CSMTPclient
end



class CPOP3client < INETclient

  def initialize( rc )
    addr = rc['address']
    port = (rc['port'] || 110).to_i

    @account = rc[ 'account' ]
    @password = rc[ 'password' ]

    super addr, port
  end

  def session_type
    Net::POPSession
  end

  def exec
    if_nopassword unless @password && @account

    do_exec
  end

  
  private


  def do_login
    @mails = []
    @session.each do |m|
      @mails.push POPmail.new(m)
    end
    @mails.freeze
  end

  def do_logout
    @mails.each do |item| item.do_proc end
    @mails = nil
  end


  def if_nopassword
    raise ArgumentError, "POP password is not given"
  end

  def if_loggedin
    @session.each do |popmail|
      port = CORE.mailset.mkmail( CORE.mailset.inbox ){|wp|
        popmail.mail wp
      }
      if_popmail port
      popmail.delete
    end
  end

  def if_popmail( port )
    port.unread = true
  end

end

class POP3client < CPOP3client
end


class APOPclient < POP3client

  def session_type
    Net::APOPSession
  end

end


end   # module EST
